<?php
/**************************************************
  Coppermine 1.5.x Plugin - maskurl
**************************************************/

/* Configuration variables
      'plugin_maskurl_mask_url'
      'plugin_maskurl_encrypt_algorithm'                                        // encryption algorithm - set to 'rijndael-128' by install
      'plugin_maskurl_encrypt_mode'                                             // encryption mode - set to 'cfb' by install
      'plugin_maskurl_encrypt_refresh'                                          // not stored in DB, trigger to refresh encryption keys
      'plugin_maskurl_windows'
      'plugin_maskurl_url_external'
   Set via installer - not directly set
      'plugin_maskurl_version'                                                  // set via installer
      'plugin_maskurl_encrypt_init'                                             // set via installer or refresh of keys when initialization completes
      'plugin_maskurl_encrypt_key'                                              // set via installer and 'Refresh Encryption Keys' request
      'plugin_maskurl_encrypt_iv'                                               // set via installer and 'Refresh Encryption Keys' request
   Not maintained via config page:
      'plugin_maskurl_mask_url_prefix'                                          // static url prefix to retrieve image via coppermine
      'plugin_maskurl_mask_url_prefix_ext'                                      // static url prefix to retrieve image outside coppermine
      'plugin_maskurl_mask_debug'                                               // causes image display to fail to provide debug - use in test environments only
*/

// create Inspekt supercage
$superCage = Inspekt::makeSuperCage();

// Make sure that this file can't be accessed directly, but only from within the Coppermine user interface
if (!defined('IN_COPPERMINE')) {
    die('Not in Coppermine...');
}

if (!GALLERY_ADMIN_MODE) {
    cpg_die(ERROR, $lang_errors['access_denied'], __FILE__, __LINE__);
}

include_once('functions.php');                                                  // include plugin subfunctions

// get sanitized POST parameters
if ($superCage->post->keyExists('submit')) {
    //Check if the form token is valid
    if(!checkFormToken()){
        cpg_die(ERROR, $lang_errors['invalid_form_token'], __FILE__, __LINE__);
    }

  // Define the sanitization patterns
  $sanitization_array = array(
      'plugin_maskurl_mask_url' => array('type' => 'checkbox', 'min' => '0', 'max' => '3'),                             // radio - 0|1|2
      'plugin_maskurl_windows' => array('type' => 'checkbox', 'min' => '0', 'max' => '1'),                              // checkbox - yes|no
      'plugin_maskurl_url_external' => array('type' => 'checkbox', 'min' => '0', 'max' => '1'),                         // checkbox - yes|no
      'plugin_maskurl_encrypt_algorithm'  => array('type' => 'raw', 'regex' => '/([^n]*+[^V\.]++)*+/'),                 // text - A-Z a-z - _ . /
      'plugin_maskurl_encrypt_mode'  => array('type' => 'raw', 'regex' => '/([^n]*+[^V\.]++)*+/'),                      // text - A-Z a-z - _ . /
      'plugin_maskurl_encrypt_refresh' => array('type' => 'trigger', 'min' => '0', 'max' => '1'),                       // checkbox - yes|no
  );
  $config_changes_counter = 0;
  $plugin_maskurl_encrypt_refresh = FALSE;
  foreach ($sanitization_array as $san_key => $san_value) {
      if (isset($CONFIG[$san_key]) == TRUE) { // only loop if config value is set --- start
          if ($san_value['type'] == 'checkbox') { // type is checkbox --- start
            if ($superCage->post->getInt($san_key) <= $san_value['max'] && $superCage->post->getInt($san_key) >= $san_value['min'] && $superCage->post->getInt($san_key) != $CONFIG[$san_key]) {   // **GMC
                $CONFIG[$san_key] = $superCage->post->getInt($san_key);
                cpg_db_query("UPDATE {$CONFIG['TABLE_CONFIG']} SET value='{$CONFIG[$san_key]}' WHERE name='$san_key'");
                $config_changes_counter++;
            } elseif($superCage->post->keyExists($san_key) != TRUE && $CONFIG[$san_key] != '0') {
                $CONFIG[$san_key] = 0;
                cpg_db_query("UPDATE {$CONFIG['TABLE_CONFIG']} SET value='{$CONFIG[$san_key]}' WHERE name='$san_key'");
                $config_changes_counter++;
            }
          } // type is checkbox --- end
          if ($san_value['type'] == 'int') { // type is integer --- start
              if ($superCage->post->getInt($san_key) <= $san_value['max'] && $superCage->post->getInt($san_key) >= $san_value['min'] && $superCage->post->getInt($san_key) != $CONFIG[$san_key]) {
                  $CONFIG[$san_key] = $superCage->post->getInt($san_key);
                  cpg_db_query("UPDATE {$CONFIG['TABLE_CONFIG']} SET value='{$CONFIG[$san_key]}' WHERE name='$san_key'");
                  $config_changes_counter++;
              }
          } // type is integer --- end
          if ($san_value['type'] == 'raw') { // type is raw --- start
              if (isset($san_value['regex']) == TRUE && preg_match($san_value['regex'], $superCage->post->getRaw($san_key)) && $superCage->post->getRaw($san_key) != $CONFIG[$san_key]) {
                  $CONFIG[$san_key] = $superCage->post->getRaw($san_key);
                  if ($superCage->post->getRaw($san_key) == 'none') {
                    $CONFIG[$san_key] = '';
                  }
                  cpg_db_query("UPDATE {$CONFIG['TABLE_CONFIG']} SET value='{$CONFIG[$san_key]}' WHERE name='$san_key'");
                  // special code for maskurl - based on key here - may need to force encrypt_refresh
                  if ($san_key == 'plugin_maskurl_encrypt_algorithm' || $san_key == 'plugin_maskurl_encrypt_mode') {
                    $plugin_maskurl_encrypt_refresh = TRUE;                     // algorithm or mode changed... need to refresh keys
                  } // end special code for maskurl
                  $config_changes_counter++;
              }
          } // type is raw --- end
          if ($san_value['type'] == 'array') { // type is array --- start              
          $evaluate_value = $superCage->post->getRaw($san_key);
              if (is_array($evaluate_value) && isset($san_value['regex']) == TRUE && isset($san_value['delimiter']) == TRUE) {
                  $temp = '';
                  for ($i = 0; $i <= count($evaluate_value); $i++) {
                      if (preg_match($san_value['regex'], $evaluate_value[$i])) {
                          $temp .= $evaluate_value[$i] . $san_value['delimiter'];
                      }
                  }
                  unset($evaluate_value);
                  $evaluate_value = rtrim($temp, $san_value['delimiter']);
                  unset($temp);
              }
              if ($evaluate_value != $CONFIG[$san_key]) {
                  $CONFIG[$san_key] = $evaluate_value;
                  cpg_db_query("UPDATE {$CONFIG['TABLE_CONFIG']} SET value='{$CONFIG[$san_key]}' WHERE name='$san_key'");
                  $config_changes_counter++;
              }
          } // type is array --- end
      } // only loop if config value is set --- end
      if ($san_value['type'] == 'trigger') { // type is trigger --- start
        // unique processing based on key.. used for 'triggers' - not stored in DB (outside of 'if config value is set')
        // drives other config updates.
        if ($superCage->post->getInt($san_key) == $san_value['max'] || ($san_key == 'plugin_maskurl_encrypt_refresh' && $plugin_maskurl_encrypt_refresh)) {
            // $CONFIG[$san_key] = $san_value['max'];
            switch ($san_key) {
              case 'plugin_maskurl_encrypt_refresh':
                $encrypt_keys = maskurl_encrypt_keys($key, $iv);
                if ($encrypt_keys) {
                  cpg_db_query("UPDATE {$CONFIG['TABLE_CONFIG']} SET value='$key}' WHERE name='plugin_maskurl_encrypt_key'");
                  cpg_db_query("UPDATE {$CONFIG['TABLE_CONFIG']} SET value='$iv}' WHERE name='plugin_maskurl_encrypt_iv'");
                  cpg_db_query("UPDATE {$CONFIG['TABLE_CONFIG']} SET `value` = '1' WHERE `name` = 'plugin_maskurl_encrypt_init'");  // update encryption initialized
                  $CONFIG['plugin_maskurl_encrypt_init'] = 1;                   // needed below to show we completed init
                  $config_changes_counter++;
                }
              break;
            }
        }
      } // type is trigger --- end
  }
}


// display config page
// get language files
require "./plugins/maskurl/lang/english.php";
if ($CONFIG['lang'] != 'english' && file_exists("./plugins/maskurl/lang/{$CONFIG['lang']}.php")) {
    require "./plugins/maskurl/lang/{$CONFIG['lang']}.php";
}
// Set the option output stuff 
if ($CONFIG['plugin_maskurl_mask_url'] == '0') {
    $option_output['plugin_maskurl_mask_url'][0] = 'checked="checked"';
    $option_output['plugin_maskurl_mask_url'][1] = '';
    $option_output['plugin_maskurl_mask_url'][2] = '';
    $option_output['plugin_maskurl_mask_url'][3] = '';
} elseif ($CONFIG['plugin_maskurl_mask_url'] == '1') {
    $option_output['plugin_maskurl_mask_url'][0] = '';
    $option_output['plugin_maskurl_mask_url'][1] = 'checked="checked"';
    $option_output['plugin_maskurl_mask_url'][2] = '';
    $option_output['plugin_maskurl_mask_url'][3] = '';
} elseif ($CONFIG['plugin_maskurl_mask_url'] == '2') {
    $option_output['plugin_maskurl_mask_url'][0] = '';
    $option_output['plugin_maskurl_mask_url'][1] = '';
    $option_output['plugin_maskurl_mask_url'][2] = 'checked="checked"';
    $option_output['plugin_maskurl_mask_url'][3] = '';
} elseif ($CONFIG['plugin_maskurl_mask_url'] == '3') {
    $option_output['plugin_maskurl_mask_url'][0] = '';
    $option_output['plugin_maskurl_mask_url'][1] = '';
    $option_output['plugin_maskurl_mask_url'][2] = '';
    $option_output['plugin_maskurl_mask_url'][3] = 'checked="checked"';
}

if ($CONFIG['plugin_maskurl_windows'] == '1') {
    $option_output['plugin_maskurl_windows'] = 'checked="checked"';
} else { 
    $option_output['plugin_maskurl_windows'] = '';
}

if ($CONFIG['plugin_maskurl_url_external'] == '1') {
    $option_output['plugin_maskurl_url_external'] = 'checked="checked"';
} else { 
    $option_output['plugin_maskurl_url_external'] = '';
}

// following option always displayed as 0 - value not stored in DB - used as a trigger
$option_output['plugin_maskurl_encrypt_refresh'] = '';

pageheader(sprintf($lang_plugin_maskurl['configure_x'], $lang_plugin_maskurl['plugin_name']));
list($timestamp, $form_token) = getFormToken();
echo <<< EOT
<form action="index.php?file=maskurl/admin" method="post" name="maskurl_settings">
EOT;
starttable('100%', sprintf("{$lang_plugin_maskurl['configure_x']} V{$CONFIG['plugin_maskurl_version']}", $lang_plugin_maskurl['plugin_name']), 3, 'cpg_zebra');

// add condition to be sure the submit button was for config updates before displaying messagebox
if ($superCage->post->keyExists('submit') && $superCage->post->getRaw('submit') == $lang_plugin_maskurl['submit']) {
    if ($config_changes_counter > 0) {
        echo <<< EOT
    <tr>
        <td class="tablef" colspan="2" >
EOT;
        msg_box('', $lang_plugin_maskurl['update_success'], '', '', 'success');
    } else {
        msg_box('', $lang_plugin_maskurl['no_changes'], '', '', 'validation');
    }
        echo <<< EOT
        </td>
    </tr>
EOT;
}

// setup of variable data needed below
$plugin_maskurl_algorithms = mcrypt_list_algorithms();
$plugin_maskurl_modes = mcrypt_list_modes();
$plugin_maskurl_result = cpg_db_query("SELECT pid, filepath, filename FROM {$CONFIG['TABLE_PICTURES']} ORDER BY pid DESC LIMIT 1;");                             // obtain picture data
$plugin_maskurl_picdata = cpg_db_fetch_row($plugin_maskurl_result);
$plugin_maskurl_defaulturl = "{$CONFIG['fullpath']}{$plugin_maskurl_picdata['filepath']}{$plugin_maskurl_picdata['filename']}";
$plugin_maskurl_currenturl = maskurl_set_url($plugin_maskurl_defaulturl);
if (strpos($CONFIG['plugin_maskurl_url_prefix'], '?') === FALSE) {
  $plugin_maskurl_connector =  '?';
} else {
  $plugin_maskurl_connector =  '&';
}

echo <<< EOT
    <!-- insert config option form code start -->
    <tr>
      <td style="vertical-align:top;">
        {$lang_plugin_maskurl['mask_url']}
      </td>
      <td>
        <input type="radio" name="plugin_maskurl_mask_url" id="plugin_maskurl_mask_url_0" class="radio" value="0" {$option_output['plugin_maskurl_mask_url'][0]} />
        <label for="plugin_maskurl_mask_url_0" class="clickable_option">{$lang_plugin_maskurl['cleartext']}</label><br />
        <input type="radio" name="plugin_maskurl_mask_url" id="plugin_maskurl_mask_url_1" class="radio" value="1" {$option_output['plugin_maskurl_mask_url'][1]} />
        <label for="plugin_maskurl_mask_url_1" class="clickable_option">{$lang_plugin_maskurl['mask']}</label><br />
EOT;
if ($CONFIG['plugin_maskurl_encrypt_init']) {
  echo <<< EOT
        <input type="radio" name="plugin_maskurl_mask_url" id="plugin_maskurl_mask_url_2" class="radio" value="2" {$option_output['plugin_maskurl_mask_url'][2]} />
        <label for="plugin_maskurl_mask_url_2" class="clickable_option">{$lang_plugin_maskurl['encrypt']}</label><br />
        <input type="radio" name="plugin_maskurl_mask_url" id="plugin_maskurl_mask_url_3" class="radio" value="3" {$option_output['plugin_maskurl_mask_url'][3]} />
        <label for="plugin_maskurl_mask_url_3" class="clickable_option">{$lang_plugin_maskurl['dyn_encrypt']}</label> ({$lang_plugin_maskurl['dyn_encrypt_comment']})<br />
EOT;
} else {
  msg_box('', $lang_plugin_maskurl['error_encrypt'], '', '', 'validation');
  $CONFIG['plugin_maskurl_debug'] = 1;
}
echo <<< EOT
      </td>
    </tr>

    <tr>
      <td>
        <label for="plugin_maskurl_encrypt_algorithm" class="clickable_option">{$lang_plugin_maskurl['encrypt_algorithm']}</label>
      </td>
      <td>
        <select name="plugin_maskurl_encrypt_algorithm" id="plugin_maskurl_encrypt_algorithm">
EOT;
          foreach ($plugin_maskurl_algorithms as $plugin_maskurl_cipher) {
            $selected = ($plugin_maskurl_cipher == $CONFIG['plugin_maskurl_encrypt_algorithm']) ? 'selected="selected"' : '';
            echo "<option value=\"$plugin_maskurl_cipher\" $selected>$plugin_maskurl_cipher</option>";
          }

echo <<< EOT
        </select>  {$lang_plugin_maskurl['encrypt_algorithm_comment']}
      </td>
    </tr>

    <tr>
      <td>
        <label for="plugin_maskurl_encrypt_mode" class="clickable_option">{$lang_plugin_maskurl['encrypt_mode']}</label>
      </td>
      <td>
        <select name="plugin_maskurl_encrypt_mode" id="plugin_maskurl_encrypt_mode">
EOT;
          foreach ($plugin_maskurl_modes as $plugin_maskurl_mode) {
            $selected = ($plugin_maskurl_mode == $CONFIG['plugin_maskurl_encrypt_mode']) ? 'selected="selected"' : '';
            echo "<option value=\"$plugin_maskurl_mode\" $selected>$plugin_maskurl_mode</option>";
          }

echo <<< EOT
        </select> {$lang_plugin_maskurl['encrypt_mode_comment']}
      </td>
    </tr>

    <tr>
      <td>
        <label for="plugin_maskurl_encrypt_refresh" class="clickable_option">{$lang_plugin_maskurl['encrypt_refresh']}</label>
      </td>
      <td>
        <input type="checkbox" name="plugin_maskurl_encrypt_refresh" id="plugin_maskurl_encrypt_refresh" class="checkbox" value="1" {$option_output['plugin_maskurl_encrypt_refresh']} />
        <label for="plugin_maskurl_encrypt_refresh" class="clickable_option">{$lang_common['yes']}</label>
      </td>
    </tr>

    <tr>
      <td>
        <label for="plugin_maskurl_windows" class="clickable_option">{$lang_plugin_maskurl['windows']}</label>
      </td>
      <td>
        <input type="checkbox" name="plugin_maskurl_windows" id="plugin_maskurl_windows" class="checkbox" value="1" {$option_output['plugin_maskurl_windows']} />
        <label for="plugin_maskurl_windows" class="clickable_option">{$lang_common['yes']}</label> ({$lang_plugin_maskurl['windows_comment']})
      </td>
    </tr>

    <tr>
      <td>
        <label for="plugin_maskurl_url_external" class="clickable_option">{$lang_plugin_maskurl['url_external']}</label>
      </td>
      <td>
        <input type="checkbox" name="plugin_maskurl_url_external" id="plugin_maskurl_url_external" class="checkbox" value="1" {$option_output['plugin_maskurl_url_external']} />
        <label for="plugin_maskurl_url_external" class="clickable_option">{$lang_common['yes']}</label> ({$lang_plugin_maskurl['url_external_comment']})
      </td>
    </tr>

    <!-- insert config option form code end -->
    <tr>
        <td class="tablef" colspan="3">
            <input type="hidden" name="form_token" value="{$form_token}" />
            <input type="hidden" name="timestamp" value="{$timestamp}" />
            <button type="submit" class="button" name="submit" value="{$lang_plugin_maskurl['submit']}">{$lang_plugin_maskurl['submit']}</button>
        </td>
    </tr>
EOT;

endtable();

// get sanitized POST parameters
if ($superCage->post->keyExists('submit')) {
    //Check if the form token is valid
    if(!checkFormToken()){
        cpg_die(ERROR, $lang_errors['invalid_form_token'], __FILE__, __LINE__);
    }
    // documentation options
    $plugin_maskurl_doc_readme = ($superCage->post->getRaw('submit') == $lang_plugin_maskurl['view_readme']) ? TRUE : FALSE;   // See if we are showing doc...
    if ($plugin_maskurl_doc_readme) {
      include ('./plugins/maskurl/readme.php');
    }
}

starttable('100%', sprintf("{$lang_plugin_maskurl['execute_x']} V{$CONFIG['plugin_maskurl_version']}", $lang_plugin_maskurl['plugin_name']), 3, 'cpg_zebra');
// sample urls - standard and based on current settings
echo <<< EOT
    <tr>
      <td valign="top">
        {$lang_plugin_maskurl['sampleurl']}<br />
        &nbsp;&nbsp;{$lang_plugin_maskurl['defaulturl']}:<br />
        &nbsp;&nbsp;{$lang_plugin_maskurl['currenturl']}:
      </td>
      <td>
        &nbsp;<br>
        {$plugin_maskurl_defaulturl}<br />
        {$plugin_maskurl_currenturl}
      </td>
    </tr>
EOT;

// debug additional info - available ciphers and modes
if ($CONFIG['plugin_maskurl_debug']) {
  // current settings
  echo <<< EOT
    <tr>
      <td valign="top">
        {$lang_plugin_maskurl['encrypt_settings']}<br />
      </td>
      <td>
        {$lang_plugin_maskurl['encrypt_algorithm']}: {$CONFIG['plugin_maskurl_encrypt_algorithm']}<br />
        {$lang_plugin_maskurl['encrypt_mode']}: {$CONFIG['plugin_maskurl_encrypt_mode']}
      </td>
    </tr>
EOT;
  // available ciphers
  echo <<< EOT
    <tr>
      <td valign="top">
        {$lang_plugin_maskurl['ciphers']}<br />
      </td>
      <td>
EOT;
    foreach ($plugin_maskurl_algorithms as $plugin_maskurl_cipher) {
        echo "$plugin_maskurl_cipher<br />\n";
    }
echo <<< EOT
      </td>
    </tr>
EOT;
// available modes
echo <<< EOT
    <tr>
      <td valign="top">
        {$lang_plugin_maskurl['modes']}<br />
      </td>
      <td>
EOT;
    foreach ($plugin_maskurl_modes as $plugin_maskurl_mode) {
        echo "$plugin_maskurl_mode<br />\n";
    }
echo <<< EOT
      </td>
    </tr>
EOT;
}
// documentation options
echo <<< EOT
    <tr>                                
        <td class="tablef" colspan="3"> 
            &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; Documentation: &nbsp;&nbsp;
            <button type="submit" class="button" name="submit" value="{$lang_plugin_maskurl['view_readme']}" >{$lang_plugin_maskurl['view_readme']}</button>
            <a href="http://forum.coppermine-gallery.net/index.php/topic,78168.0.html" class="admin_menu"> {$lang_plugin_maskurl['announcement']} </a>
        </td>
    </tr>
EOT;
endtable();

echo <<< EOT
</form>
EOT;
pagefooter();
?>