<?php
/**************************************************
  Coppermine 1.5.x Plugin - maskurl
**************************************************/

// Make sure that this file can't be accessed directly, but only from within the Coppermine user interface
if (!defined('IN_COPPERMINE')) {
    die('Not in Coppermine...');
}

include_once('functions.php');                                                  // include plugin subfunctions

// Define plugin actions here
$thisplugin->add_action('plugin_install','maskurl_install');
$thisplugin->add_action('plugin_uninstall','maskurl_uninstall');
$thisplugin->add_action('page_start','maskurl_page_start');                     // page start intercept to set up encryption

// Define plugin filters here
$thisplugin->add_filter('picture_url','maskurl_url');                           // picture_url intercept used to mask URL for all sizes

// Define the plugin's functions here
function maskurl_install() {
    global $CONFIG;
    $maskurl_version = '1.1';
    $maskurl_encrypt_algorithm = 'rijndael-128';
    $maskurl_encrypt_mode = 'cfb';
    // Add the config options for the plugin
    cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_maskurl_version', '$maskurl_version')");
    cpg_db_query("UPDATE {$CONFIG['TABLE_CONFIG']} SET `value` = '$maskurl_version' WHERE `name` = 'plugin_maskurl_version'");  // update version if re-install
    cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_maskurl_mask_url', '0')");
    cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_maskurl_encrypt_algorithm', '$maskurl_encrypt_algorithm')");
    cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_maskurl_encrypt_mode', '$maskurl_encrypt_mode')");
    cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_maskurl_encrypt_init', '0')");
    cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_maskurl_encrypt_key', '')");
    cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_maskurl_encrypt_iv', '')");
    cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_maskurl_windows', '0')");
    // See if webserver is Windows OS...
    $maskurl_windows = (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') ? '1' : '0';
    cpg_db_query("UPDATE {$CONFIG['TABLE_CONFIG']} SET `value` = '$maskurl_windows' WHERE `name` = 'plugin_maskurl_windows'");  // update OS info to match current system
    cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_maskurl_url_external', '0')");
    // hidden options not supported by config page
    cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_maskurl_url_prefix', 'index.php?file=maskurl/displayimage')");
    cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_maskurl_url_prefix_ext', 'plugins/maskurl/displayimage.php')");
    cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_maskurl_debug', '0')");
    // add $CONFIG values needed to complete installation
    $CONFIG['plugin_maskurl_windows'] = $maskurl_windows;                       // needed by maskurl_encrypt_keys function call
    $CONFIG['plugin_maskurl_encrypt_algorithm'] = $maskurl_encrypt_algorithm;   // needed by maskurl_encrypt_keys function call
    $CONFIG['plugin_maskurl_encrypt_mode'] = $maskurl_encrypt_mode;             // needed by maskurl_encrypt_keys function call
    $encrypt_keys = maskurl_encrypt_keys($key, $iv);
    if ($encrypt_keys) {
      cpg_db_query("UPDATE {$CONFIG['TABLE_CONFIG']} SET `value` = '$key' WHERE `name` = 'plugin_maskurl_encrypt_key'"); // update encryption key
      cpg_db_query("UPDATE {$CONFIG['TABLE_CONFIG']} SET `value` = '$iv' WHERE `name` = 'plugin_maskurl_encrypt_iv'");   // update encryption init vector
      cpg_db_query("UPDATE {$CONFIG['TABLE_CONFIG']} SET `value` = '1' WHERE `name` = 'plugin_maskurl_encrypt_init'");   // update encryption initialized
      return true;
    }
    else {
      return true;
    }
}


// function to modify pic_row - alter url to call our display page for img to mask URL in html source
function maskurl_page_start() {
    global $CONFIG;
    // if not doing dynamic encryption - then return...
    if ($CONFIG["plugin_maskurl_mask_url"] != 3) {
      return;
    }
    // set up dynamic encryption - encrypt key will come from database (loaded into $CONFIG) - set up the Init Vector here
    $encrypt_keys = maskurl_encrypt_keys($key, $iv);
    if ($encrypt_keys) {
      $CONFIG["plugin_maskurl_encrypt_iv"] = $iv;                               // set Init Vector in $CONFIG
      setcookie($CONFIG['cookie_name'] . "_maskurl", $iv);                      // store it in cookie
    } else {
      // shouldn't happen - error generating encryption iv..
    }
}


// function to modify pic_row - alter url to call our display page for img to mask URL in html source
function maskurl_url($pic_row) {
    global $CONFIG;
    // if not masking - then return...
    if (!$CONFIG["plugin_maskurl_mask_url"]) {
      return $pic_row;
    }
    // is this request for a coppermine provided image (such as images/thumbs/thumb_nopic.png when no pics are in an album...)
    if ($pic_row['system_icon'] || substr($pic_row["url"], 0, 7) == 'images/') {
      // 'system_icon' should catch them all - but check path to be safe (calls from other plugins may not know it is system image).
      return $pic_row;
    }
    // Work to do - we need to change our prefix snd mask or encrypt if requested.
    $pic_row["url"] = maskurl_set_url($pic_row["url"]);
    return $pic_row;
}


function maskurl_uninstall() {
    global $CONFIG, $lang_errors;
    $superCage = Inspekt::makeSuperCage();
    if (!checkFormToken()) {
        cpg_die(ERROR, $lang_errors['invalid_form_token'], __FILE__, __LINE__);
    }
    // Delete the plugin config records
    cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_maskurl_version'");
    cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_maskurl_mask_url'");
    cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_maskurl_encrypt_algorithm'");
    cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_maskurl_encrypt_mode'");
    cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_maskurl_encrypt_init'");
    cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_maskurl_encrypt_key'");
    cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_maskurl_encrypt_iv'");
    cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_maskurl_windows'");
    cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_maskurl_url_external'");
    cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_maskurl_url_prefix'");
    cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_maskurl_url_prefix_ext'");
    cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_maskurl_debug'");
    return true;
}
?>