<?php
/**************************************************
  Coppermine 1.5.x Plugin - maskurl
**************************************************/
// Serve requested image file - decrypt/decode path to image if needed - watermark if requested.

// Can be executed inside or outside of Coppermine. Referenced in <img> tag in Coppermine html with plugin active and configured.
// execute within coppermine:  index.php?file=maskurl/displayimage&{photokey or photopath specification}
// execute outside coppermine: plugins/maskurl/displayimage.php?{photokey or photopath specification}

// Serve the requested photo from protected album filesystem (no htaccess required for this method).
// file name info is passed as $photokey (obfuscated) or $photopath (clear text)

// set up diagnostics
$debug_trace = array();
$debug_error = array();

// see if we are inside Coppermine
if (defined('IN_COPPERMINE')) {
  // executed within Coppermine - a lot of work done for us...
  // * Inspekt/SuperCage already set up
  // * CONFIG array populated
  // * Database connection made
  $debug_trace[] = 'MaskUrl display image invoked inside Coppermine';

  // capture GET data required (validated later...)
  $photopath = $superCage->get->getRaw('photopath');
  $photokey = $superCage->get->getRaw('photokey');

  // set constants - execution is in Coppermine Root directory
  $includeprefix = "plugins/maskurl/";                                          // prefix for code includes
  $fileprefix = '';                                                             // prefix for access to albums directory
  // see if we need cookie information for dynamic encryption
  if ($CONFIG["plugin_maskurl_mask_url"] == 3) {
    $debug_trace[] = 'Dynamic Encryption - loading IV';
    if ($superCage->cookie->keyExists($CONFIG['cookie_name'] . "_maskurl")) {
      $CONFIG["plugin_maskurl_encrypt_iv"] = $superCage->cookie->getRaw($CONFIG['cookie_name'] . "_maskurl");
      $debug_trace[] = "IV: {$CONFIG["plugin_maskurl_encrypt_iv"]}";
    }
  }
} else {
  // executed outside of Coppermine... get the extra support info we need
  require ('displayimage_cpg_inc.php');
}

// set up sample (and error) pictures
$sampleimage = 'plugins/maskurl/sample.jpg';
$samplethumbimage = 'plugins/maskurl/sample_thumb.jpg';
// local functions
require_once ("{$includeprefix}functions.php");
require "{$includeprefix}lang/english.php";                                     // include english language file
if ($CONFIG['lang'] != 'english' && file_exists("$includeprefix.lang/{$CONFIG['lang']}.php")) {
  require "{$includeprefix}lang/{$CONFIG['lang']}.php";                         // include config indicated language file if available
}

// diagnostics trace
$debug_trace[] = "Input: Photopath: $photopath - Key: $photokey";
$debug_trace[] = "Config Fullpath: {$CONFIG["fullpath"]} - Config Thumb Prefix: {$CONFIG["thumb_pfx"]}";

// if photopath not provided, extract information from key.
if (!$photopath) {
  $debug_trace[] = "Processing key: $photokey";
  if (!$photokey) {
    $debug_error[] = $lang_plugin_maskurl['error_no_input'];
  } else {
    // decode key
    if ($CONFIG["plugin_maskurl_mask_url"] == '1') {
      $photopath = maskurl_deobfuscate($photokey);
    }
    elseif ($CONFIG["plugin_maskurl_mask_url"] == '2' || $CONFIG["plugin_maskurl_mask_url"] == '3') {
      $photopath = maskurl_decrypt($photokey);
    }
    $debug_trace[] = "Generated Photopath Prefix: ".substr($photopath, 0, 10);
  }
}

// now have photopath - either directly from _GET or from encoded/encrypted key from _GET
// some sanity validations before using
$validpath = TRUE;
// does path match config path to albums? or our sample pictures (used by plugin configuration)
$compare = substr_compare("$photopath", $CONFIG["fullpath"], 0, strlen($CONFIG["fullpath"]));
if ($compare != '0') {
  // no match to config...
  $validpath = FALSE;
  $debug_error[] = $lang_plugin_maskurl['error_dir_mismatch'];
}
// does this request attempt to 'back up' the directory path?
elseif (strpos($photopath,'../')) {
  // attempt to backup the directory path...
  $validpath = FALSE;
  $debug_error[] = $lang_plugin_maskurl['error_invalid_char'];
}

// have valid input - go do the work
if ($validpath) {
  $filename = "{$fileprefix}$photopath";
}
else {
  // load an error graphic...
  $filename = "{$fileprefix}$sampleimage";
}

// determine image type and output
$extension = substr(strrchr($filename, "."), 1);
$sql = "SELECT mime FROM {$CONFIG['TABLE_FILETYPES']} WHERE extension = '$extension' LIMIT 1;";
$result = cpg_db_query($sql, $db);                                              // obtain mime type
$row = cpg_db_fetch_row($result);
$debug_trace[] = "File extension: $extension - Mime type: " . $row['mime'];
if (!$row) {
  $debug_error[] = $lang_plugin_maskurl['error_filetype'];
}

if ($CONFIG['plugin_maskurl_debug']) {
  // if config debug set - echo debug/error information... We will be unable to output file as headers will fail with previous echo..
  // only set this option in test galleries - and view output entering script directly: http://cpgurl/plugins/maskurl/displayimage.php?xxxxxxxxx=yyyyyyyyyyyy
  // where xxxxxxxxx=yyyyyyyyyyyy is appropriate clear text (photopath=), or obfuscated or encrypted key (photokey=).
  // We can't echo data if script is called from <img> tag
  // debug must be manually set in database - not supported in admin dialog
  echo "Debug Trace: <pre>".print_r($debug_trace, TRUE)."</pre><br />";
  echo "Debug Error: <pre>".print_r($debug_error, TRUE)."</pre><br />";
} else {
  header('Content-Type: ' . $row['mime']);
  header('Content-Length: ' . filesize($filename));
  readfile($filename);
}
?>