<?php
/**************************************************
  Coppermine 1.5.x Plugin - maskurl
**************************************************/

// Make sure that this file can't be accessed directly, but only from within the Coppermine user interface
if (!defined('IN_COPPERMINE')) {
    die('Not in Coppermine...');
}

// base install, uninstall, and direct plugin called functions are in codebase.php
// sub functions may be shared between codebase, admin, and viewgalleryphoto pages and reside here.

// Not intended for security or one-way encryption, but when a string needs to somewhat 'hidden' for email/display and be reversed later...
function maskurl_obfuscate($string) {
    return str_rot13( base64_encode( $string ) );
}


// unmask string obfuscated above
function maskurl_deobfuscate($string) {
    return base64_decode( str_rot13( $string ) );
}


// When stronger encryption is desired - at the expense of CPU time...

// generate encryption keys and initialization vector
function maskurl_encrypt_keys(&$key, &$iv) {
    global $CONFIG;
    // generate encryption key and initization vector
    if (function_exists('mcrypt_get_key_size') &&
        in_array($CONFIG['plugin_maskurl_encrypt_algorithm'], mcrypt_list_algorithms()) &&
        in_array($CONFIG['plugin_maskurl_encrypt_mode'], mcrypt_list_modes())) {
      // encryption functions, requested algorithm, and mode available..
//      $key_size = mcrypt_get_key_size(MCRYPT_RIJNDAEL_128, MCRYPT_MODE_CFB);
      $key_size = mcrypt_get_key_size($CONFIG['plugin_maskurl_encrypt_algorithm'], $CONFIG['plugin_maskurl_encrypt_mode']);
      $encryption_key = openssl_random_pseudo_bytes($key_size, $strong);
      $key = base64_encode( $encryption_key );
//      $iv_size = mcrypt_get_iv_size(MCRYPT_RIJNDAEL_128, MCRYPT_MODE_CFB);
      $iv_size = mcrypt_get_iv_size($CONFIG['plugin_maskurl_encrypt_algorithm'], $CONFIG['plugin_maskurl_encrypt_mode']);
      if ($CONFIG['plugin_maskurl_windows']) {
        // Microsoft Windows needs something special if PHP < 5.3... Since 1.5 still supports PHP V4 - we'll play it safe...
        srand();
        $init_vector = mcrypt_create_iv($iv_size, MCRYPT_RAND); // 16 bytes output
      } else {
        // all other OS (and can be used for Windows if PHP is 5.3 or higher)
        // from PHP Doc for mcrypt_create_iv Changelog: http://php.net/manual/en/function.mcrypt-create-iv.php
        // 5.3.0   MCRYPT_DEV_RANDOM and MCRYPT_DEV_URANDOM became available on Windows platforms.
        // 5.3.0   It is no longer required to call srand() first. This is now done automatically.
        $init_vector = mcrypt_create_iv($iv_size, MCRYPT_DEV_URANDOM); // 16 bytes output
      }
      $iv = base64_encode( $init_vector );
      // $strong will be true if the key is crypto safe - should always be the case...
    } else {
      // encryption not installed - or algoritm/mode missing??
      $strong = FALSE;
    }
    if ($strong) {
      return TRUE;
    } else {
      return FALSE;
    }
}


// encrypt and encode the provided string
function maskurl_encrypt($string, $format = 'urlencode') {
    global $CONFIG;
    $encrypt_key = base64_decode( $CONFIG['plugin_maskurl_encrypt_key'] );
    $encrypt_iv = base64_decode( $CONFIG['plugin_maskurl_encrypt_iv'] );
//    $encrypted_string = mcrypt_encrypt(MCRYPT_RIJNDAEL_128, $encrypt_key, $string, MCRYPT_MODE_CFB, $encrypt_iv);
    $encrypted_string = mcrypt_encrypt($CONFIG['plugin_maskurl_encrypt_algorithm'], $encrypt_key, $string, $CONFIG['plugin_maskurl_encrypt_mode'], $encrypt_iv);
    $encrypted_string = base64_encode( $encrypted_string );
    if ($format == 'urlencode') {
      return urlencode( $encrypted_string );
    } else {
      return $encrypted_string;
    }
}


// decode and decrypt the provided string
function maskurl_decrypt($string, $format = 'base64') {
    global $CONFIG;
    $encrypt_key = base64_decode( $CONFIG['plugin_maskurl_encrypt_key'] );
    $encrypt_iv = base64_decode( $CONFIG['plugin_maskurl_encrypt_iv'] );
    if ($format == 'urlencode') {
      $string = urldecode( $string );
    }
    $encrypted_string = str_replace(' ', '+', $string);                         // urldecode replaces + with blank... need to reset.
    $encrypted_string =  base64_decode( $encrypted_string );
//    return rtrim(mcrypt_decrypt(MCRYPT_RIJNDAEL_128, $encrypt_key, $encrypted_string, MCRYPT_MODE_CFB, $encrypt_iv),"\0");
    return rtrim(mcrypt_decrypt($CONFIG['plugin_maskurl_encrypt_algorithm'], $encrypt_key, $encrypted_string, $CONFIG['plugin_maskurl_encrypt_mode'], $encrypt_iv),"\0");
}


// set new url
function maskurl_set_url($url) {
    global $CONFIG;
    $new_url = $url;                                                            // in case no changes (possible when called from admin.php)
    // convert provided url to key if needed for masking/encryption
    if ($CONFIG['plugin_maskurl_mask_url'] == '1') {
      $url_key =  maskurl_obfuscate($url);
    } elseif ($CONFIG['plugin_maskurl_mask_url'] == '2' || $CONFIG['plugin_maskurl_mask_url'] == '3') {
      $url_key =  maskurl_encrypt($url);
    }
    $plugin_maskurl_url_prefix = ($CONFIG['plugin_maskurl_url_external']) ? $CONFIG['plugin_maskurl_url_prefix_ext'] : $CONFIG['plugin_maskurl_url_prefix'];
    if (strpos($plugin_maskurl_url_prefix, '?') === FALSE) {
      $connector =  '?';
    } else {
      $connector =  '&';
    }

    // set up new URL as requested
    if ($CONFIG['plugin_maskurl_mask_url']) {
      $new_url = "$plugin_maskurl_url_prefix{$connector}photokey={$url_key}";         // mask request
    }
    return $new_url;
}


// function from Coppermine include/init.inc.php included here - as init.inc.php can only be invoked within coppermine... and does a lot we don't need.
// plugins displayimage can be invoked outside of CPG
if (!function_exists(cpgGetMicroTime)) {
  function cpgGetMicroTime() {
      list($usec, $sec) = explode(' ', microtime());
      return ((float)$usec + (float)$sec);
  }
}
?>